from machine import Pin,SPI,PWM
import framebuf
import time
import random
import math
#from array import array
import pac_felder

BL = 13
DC = 8
RST = 12
MOSI = 11
SCK = 10
CS = 9


class LCD_1inch14(framebuf.FrameBuffer):
    def __init__(self):
        self.width = 240
        self.height = 135
        
        self.cs = Pin(CS,Pin.OUT)
        self.rst = Pin(RST,Pin.OUT)
        
        self.cs(1)
        self.spi = SPI(1)
        self.spi = SPI(1,1000_000)
        self.spi = SPI(1,10000_000,polarity=0, phase=0,sck=Pin(SCK),mosi=Pin(MOSI),miso=None)
        self.dc = Pin(DC,Pin.OUT)
        self.dc(1)
        self.buffer = bytearray(self.height * self.width * 2)
        super().__init__(self.buffer, self.width, self.height, framebuf.RGB565)
        self.init_display()
        
        self.red   =   0x07E0
        self.green =   0x001f
        self.blue  =   0xf800
        self.white =   0xffff
        self.black =   0x0000
        self.yellow=   0xE0FF
        
    def write_cmd(self, cmd):
        self.cs(1)
        self.dc(0)
        self.cs(0)
        self.spi.write(bytearray([cmd]))
        self.cs(1)

    def write_data(self, buf):
        self.cs(1)
        self.dc(1)
        self.cs(0)
        self.spi.write(bytearray([buf]))
        self.cs(1)

    def init_display(self):
        """Initialize dispaly"""  
        self.rst(1)
        self.rst(0)
        self.rst(1)
        
        self.write_cmd(0x36)
        self.write_data(0x70)

        self.write_cmd(0x3A) 
        self.write_data(0x05)

        self.write_cmd(0xB2)
        self.write_data(0x0C)
        self.write_data(0x0C)
        self.write_data(0x00)
        self.write_data(0x33)
        self.write_data(0x33)

        self.write_cmd(0xB7)
        self.write_data(0x35) 

        self.write_cmd(0xBB)
        self.write_data(0x19)

        self.write_cmd(0xC0)
        self.write_data(0x2C)

        self.write_cmd(0xC2)
        self.write_data(0x01)

        self.write_cmd(0xC3)
        self.write_data(0x12)   

        self.write_cmd(0xC4)
        self.write_data(0x20)

        self.write_cmd(0xC6)
        self.write_data(0x0F) 

        self.write_cmd(0xD0)
        self.write_data(0xA4)
        self.write_data(0xA1)

        self.write_cmd(0xE0)
        self.write_data(0xD0)
        self.write_data(0x04)
        self.write_data(0x0D)
        self.write_data(0x11)
        self.write_data(0x13)
        self.write_data(0x2B)
        self.write_data(0x3F)
        self.write_data(0x54)
        self.write_data(0x4C)
        self.write_data(0x18)
        self.write_data(0x0D)
        self.write_data(0x0B)
        self.write_data(0x1F)
        self.write_data(0x23)

        self.write_cmd(0xE1)
        self.write_data(0xD0)
        self.write_data(0x04)
        self.write_data(0x0C)
        self.write_data(0x11)
        self.write_data(0x13)
        self.write_data(0x2C)
        self.write_data(0x3F)
        self.write_data(0x44)
        self.write_data(0x51)
        self.write_data(0x2F)
        self.write_data(0x1F)
        self.write_data(0x1F)
        self.write_data(0x20)
        self.write_data(0x23)
        
        self.write_cmd(0x21)

        self.write_cmd(0x11)

        self.write_cmd(0x29)

    def show(self):
        self.write_cmd(0x2A)
        self.write_data(0x00)
        self.write_data(0x28)
        self.write_data(0x01)
        self.write_data(0x17)
        
        self.write_cmd(0x2B)
        self.write_data(0x00)
        self.write_data(0x35)
        self.write_data(0x00)
        self.write_data(0xBB)
        
        self.write_cmd(0x2C)
        
        self.cs(1)
        self.dc(1)
        self.cs(0)
        self.spi.write(self.buffer)
        self.cs(1)

def pacman0(x,y,r):
    for angle in range(0, 90, 2):    # Loeschen bei Bewegung
        yr=int(r*math.sin(math.radians(angle)))
        xr=int(r*math.cos(math.radians(angle)))
        LCD.pixel(x-xr,y+yr,LCD.black)
        LCD.pixel(x-xr,y-yr,LCD.black)
        LCD.pixel(x+xr,y+yr,LCD.black)
        LCD.pixel(x+xr,y-yr,LCD.black)

def pacman1(x,y,r,farbe):   # Kreis mit Farbe
    for angle in range(0, 90, 2):    # 0 to 90 degrees in 2s
        yr=int(r*math.sin(math.radians(angle)))
        xr=int(r*math.cos(math.radians(angle)))
        LCD.pixel(x-xr,y+yr,farbe)
        LCD.pixel(x-xr,y-yr,farbe)
        LCD.pixel(x+xr,y+yr,farbe)
        LCD.pixel(x+xr,y-yr,farbe)

def pacman_rechts(x,y,r,farbe):   # Kreisausschnitt rechts schwarz
    for angle in range(0, 65, 2):    # 0 to 90 degrees in 2s
        yr=int(r*math.sin(math.radians(angle)))
        xr=int(r*math.cos(math.radians(angle)))
        LCD.pixel(x+xr,y+yr,farbe)

def pacman_links(x,y,r,farbe):   # Kreisausschnitt links schwarz
    for angle in range(0, 65, 2):    # 0 to 90 degrees in 2s
        yr=int(r*math.sin(math.radians(angle)))
        xr=int(r*math.cos(math.radians(angle)))
        LCD.pixel(x-xr,y+yr,farbe)
        

def pacman(x,y):
    for r in range(0,15):
        pacman1(25+x,25,r,LCD.yellow)
        pacman1(210-x,25+y,r,LCD.yellow)
    LCD.fill_rect(23+x,18,4,4,LCD.red)
    LCD.fill_rect(213-x,18+y,4,4,LCD.red)
    LCD.show()
    for r in range(0,15):
        pacman_rechts(25+x,25,r,LCD.black)
        pacman_links(210-x,25+y,r,LCD.black)
    LCD.show()
    for r in range(0,15):
        pacman0(210-x,25+y,r)
        pacman0(25+x,25,r)

def pac(x,y,farbe):
    for r in range(0,5):
        for angle in range(0, 90, 2):    # 0 to 90 degrees in 2s
            yr=int(r*math.sin(math.radians(angle)))
            xr=int(r*math.cos(math.radians(angle)))
            LCD.pixel(x-xr,y+yr,farbe)
            LCD.pixel(x-xr,y-yr,farbe)
            LCD.pixel(x+xr,y+yr,farbe)
            LCD.pixel(x+xr,y-yr,farbe)
def pac_rechts(x,y):
    for r in range(0,5): 
        for angle in range(0, 65, 2):    # 0 to 90 degrees in 2s
            yr=int(r*math.sin(math.radians(angle)))
            xr=int(r*math.cos(math.radians(angle)))
            LCD.pixel(x+xr,y+yr,0x0000)
def pac_links(x,y):
    for r in range(0,5): 
        for angle in range(0, 65, 2):    # 0 to 90 degrees in 2s
            yr=int(r*math.sin(math.radians(angle)))
            xr=int(r*math.cos(math.radians(angle)))
            LCD.pixel(x-xr,y-yr,0x0000)
def pac_hoch(x,y):
    for r in range(0,5): 
        for angle in range(0, 65, 2):    # 0 to 90 degrees in 2s
            yr=int(r*math.sin(math.radians(angle)))
            xr=int(r*math.cos(math.radians(angle)))
            LCD.pixel(x+xr,y-yr,0x0000)
def pac_runter(x,y):
    for r in range(0,5): 
        for angle in range(0, 65, 2):    # 0 to 90 degrees in 2s
            yr=int(r*math.sin(math.radians(angle)))
            xr=int(r*math.cos(math.radians(angle)))
            LCD.pixel(x-xr,y+yr,0x0000)

pwm = PWM(Pin(BL))
pwm.freq(1000)
pwm.duty_u16(32768)#max 65535
LCD = LCD_1inch14()
LCD.fill(LCD.black)
LCD.show()
  
def init_screen(pos,xstart,ystart,x,y):
    LCD.fill(LCD.black)
    LCD.show()
    LCD.vline(0,0,135,LCD.yellow)
    LCD.vline(239,1,135,LCD.yellow)
    pos=0
    pac_felder.pac_rinth(LCD,Liste,pos,xstart,ystart,x,y)
    pac_felder.geist1(Geist_Liste,xstart,ystart,x,y)
    LCD.text(str(leben),155,126,LCD.white)
    LCD.text(str(punkte),70,126,LCD.white)


keyA = Pin(15,Pin.IN,Pin.PULL_UP)
keyB = Pin(17,Pin.IN,Pin.PULL_UP)
key2 = Pin(2 ,Pin.IN,Pin.PULL_UP) #上
key3 = Pin(3 ,Pin.IN,Pin.PULL_UP)#中
key4 = Pin(16 ,Pin.IN,Pin.PULL_UP)#左
key5 = Pin(18 ,Pin.IN,Pin.PULL_UP)#下
key6 = Pin(20 ,Pin.IN,Pin.PULL_UP)#右

Liste = bytearray(240 * 135)
Geist_Liste = bytearray(12 * 7)
pos =0
x=0
y=0
yp=0
xstart=0
ystart=0
start=0
punkte=0
leben=3
ziel = 58
xg=4 #Startpunkt Geist
yg=3 #Startpunkt Geist
num=0
step = 0 # Geisterschritte
startzeit = 0
restzeit = 0
sec = 0

while(1):
    if start == 0:
        if x < 240:
            LCD.fill(LCD.black)
            LCD.rect(1,1,239,134,LCD.white)
            LCD.text("Pico - Man",80,10,LCD.green)
            LCD.text("by gebmedia",70,120,LCD.green)
            LCD.fill_rect(40,30,160,80,LCD.blue)
            LCD.rect(45,35,150,70,LCD.white)
            LCD.text("A-Button: Start",50,45,LCD.white)
            LCD.text("Sammle die Punkte",50,65,LCD.yellow)
            LCD.text("und meide die",60,80,LCD.yellow)
            LCD.text("Geister.",75,95,LCD.yellow)
            pacman(x,y)
            x = x + 8
            y = y + 6
        else:
            x = -20
            y = -20
    if(keyA.value() == 0):
        start=1
        punkte=0
        ziel = 58
        leben = 3
        sec = 0
        restzeit = 60
        init_screen(pos,xstart,ystart,x,y)
        fobj=open("daten.txt", 'r')
        x = int(fobj.readline())
        xe = int(fobj.readline())
        y = int(fobj.readline())
        fobj.close()
        farbe = 0xE0FF
        pac(x,y,farbe)
        LCD.show()
        # Zeit starten
        startzeit = time.ticks_ms()  
                
    if start == 1:    
        if time.ticks_ms() - startzeit > sec*1000:
            sec += 1
            restzeit -= 1
            if restzeit < 20:
                LCD.text("Zeit:",175,126,LCD.white)
                LCD.text(str(restzeit +1),220,126,LCD.black)
                LCD.text(str(restzeit),220,126,LCD.white)
            print(restzeit)
        if ziel == 0:
                xg=0
                yg=0
        if step > 105 and num == 0:
            xg=4 #Startpunkt Geist1
            yg=3 
            pac_felder.geist2(Geist_Liste,xstart,ystart,x,y)
            step = 0
            num = 1
        if step > 105 and num == 1:
            xg=4 #Startpunkt Geist1
            yg=3 
            pac_felder.geist1(Geist_Liste,xstart,ystart,x,y)
            step = 0
            num = 0
        if step < 100:
            LCD.fill_rect(xg*20 +8,yg*20 +3,4,5,LCD.green)
            LCD.fill_rect(xg*20 +6,yg*20 +8,8,5,LCD.green)
            LCD.vline(xg*20 +6,yg*20 +13,3,LCD.green)
            LCD.vline(xg*20 +13,yg*20 +13,3,LCD.green)
# hier Leben abziehen bei Geist 1
            if (x == xg*20 +10 and y == yg*20 +10) :
                LCD.text(str(leben),155,126,LCD.black)
                leben -= 1
                LCD.text(str(leben),155,126,LCD.white)
                time.sleep(1)
            # hoch
            if Geist_Liste[xg + yg*12] == 1:
                LCD.fill_rect(xg*20 +5,yg*20+2,12,14,LCD.black)
                if Liste[xg*20 + yg*20*240] == 224:
                    LCD.fill_rect(xg*20+10,yg*20+8,4,4,LCD.red)
                yg = yg -1
                step += 1
                LCD.fill_rect(xg*20 +8,yg*20 +3,4,5,LCD.green)
                LCD.fill_rect(xg*20 +6,yg*20 +8,8,5,LCD.green)
                LCD.vline(xg*20 +6,yg*20 +13,3,LCD.green)
                LCD.vline(xg*20 +13,yg*20 +13,3,LCD.green)
            # links                
            if Geist_Liste[xg + yg*12] == 2:
                LCD.fill_rect(xg*20 +5,yg*20+2,12,14,LCD.black)
                if Liste[xg*20 + yg*20*240] == 224:
                    LCD.fill_rect(xg*20+10,yg*20+8,4,4,LCD.red)
                xg = xg -1
                step += 1
                LCD.fill_rect(xg*20 +8,yg*20 +3,4,5,LCD.green)
                LCD.fill_rect(xg*20 +6,yg*20 +8,8,5,LCD.green)
                LCD.vline(xg*20 +6,yg*20 +13,3,LCD.green)
                LCD.vline(xg*20 +13,yg*20 +13,3,LCD.green)
            # runter
            if Geist_Liste[xg + yg*12] == 3:
                LCD.fill_rect(xg*20 +5,yg*20+2,12,14,LCD.black)
                if Liste[xg*20 + yg*20*240] == 224:
                    LCD.fill_rect(xg*20+10,yg*20+8,4,4,LCD.red)
                yg = yg +1
                step += 1
                LCD.fill_rect(xg*20 +8,yg*20 +3,4,5,LCD.green)
                LCD.fill_rect(xg*20 +6,yg*20 +8,8,5,LCD.green)
                LCD.vline(xg*20 +6,yg*20 +13,3,LCD.green)
                LCD.vline(xg*20 +13,yg*20 +13,3,LCD.green)
            # rechts
            if Geist_Liste[xg + yg*12] == 4:
                LCD.fill_rect(xg*20 +5,yg*20+2,12,14,LCD.black)
                if Liste[xg*20 + yg*20*240] == 224:
                    LCD.fill_rect(xg*20+10,yg*20+8,4,4,LCD.red)
                xg = xg +1
                step += 1                   
                LCD.fill_rect(xg*20 +8,yg*20 +3,4,5,LCD.green)
                LCD.fill_rect(xg*20 +6,yg*20 +8,8,5,LCD.green)
                LCD.vline(xg*20 +6,yg*20 +13,3,LCD.green)
                LCD.vline(xg*20 +13,yg*20 +13,3,LCD.green)
            LCD.show()
        else:
            LCD.fill_rect(xg*20 +5,yg*20+2,12,14,LCD.black)
            if Liste[xg*20 + yg*20*240] == 224:
                LCD.fill_rect(xg*20+10,yg*20+8,4,4,LCD.red)
            LCD.show()
        
        if(key2.value() == 0): #up
            step += 1
            pos= x+y*240
            if Liste[x-10 + (y-10)*240] == 224:
                LCD.text(str(punkte),70,126,LCD.black)
                punkte = punkte + 10
                LCD.text(str(punkte),70,126,LCD.white)
                Liste[x-10 + (y-10)*240] = 0
                ziel -=1
            if Liste[pos] == 0 and Liste[x+ (y-10)*240] != 255 :
                farbe = 0x0000
                pac(x,y,farbe)
                if y > 10:
                    y = y - 20
                farbe = 0xE0FF
                pac(x,y,farbe)
                pac_hoch(x,y)
                LCD.show()
                farbe = 0xE0FF
                pac(x,y,farbe)
                LCD.show()
            else:
                farbe = 0xE0FF
                pac(x,y,farbe)
                pac_hoch(x,y)
                LCD.show()
                farbe = 0xE0FF
                pac(x,y,farbe)
                LCD.show()
                
        if(key5.value() == 0):#down
            step += 1
            pos= x+y*240
            if Liste[x-10 + (y+10)*240] == 224:
                LCD.text(str(punkte),70,126,LCD.black)
                punkte = punkte + 10
                LCD.text(str(punkte),70,126,LCD.white)
                Liste[x-10 + (y+10)*240] = 0
                ziel -=1
            if Liste[pos] == 0 and Liste[x+ (y+10)*240] != 255 :
                farbe = 0x0000
                pac(x,y,farbe)
                if y < 120:
                    y = y + 20
                farbe = 0xE0FF
                pac(x,y,farbe)
                pac_runter(x,y)
                LCD.show()
                farbe = 0xE0FF
                pac(x,y,farbe)
                LCD.show()
            else:
                farbe = 0xE0FF
                pac(x,y,farbe)
                pac_runter(x,y)
                LCD.show()
                farbe = 0xE0FF
                pac(x,y,farbe)
                LCD.show()
            if Liste[pos] == 0 and y > 119 :
                farbe = 0x0000
                pac(x,y,farbe)
                x = 130
                y = 10
                farbe = 0xE0FF
                pac(x,y,farbe)
                pac_runter(x,y)
                LCD.show()
                farbe = 0xE0FF
                pac(x,y,farbe)
                LCD.show()
            
        if(key4.value() == 0):#left
            step += 1
            pos= x+y*240
            if Liste[x-10 + (y-10)*240] == 224:
                LCD.text(str(punkte),70,126,LCD.black)
                punkte = punkte + 10
                LCD.text(str(punkte),70,126,LCD.white)
                Liste[x-10 + (y-10)*240] = 0
                ziel -=1
            if Liste[pos] == 0 and Liste[pos-10] != 255 :
                farbe = 0x0000
                pac(x,y,farbe)
                if x > 10:
                    x = x - 20
                farbe = 0xE0FF
                pac(x,y,farbe)
                pac_links(x,y)
                LCD.show()
                farbe = 0xE0FF
                pac(x,y,farbe)
                LCD.show()
            else:
                farbe = 0xE0FF
                pac(x,y,farbe)
                pac_links(x,y)
                LCD.show()
                farbe = 0xE0FF
                pac(x,y,farbe)
                LCD.show()
            
        if(key6.value() == 0):#right
            step += 1
            pos= x+y*240
            if Liste[x+10 + (y-10)*240] == 224:
                LCD.text(str(punkte),70,126,LCD.black)
                punkte = punkte + 10
                LCD.text(str(punkte),70,126,LCD.white)
                Liste[x+10 + (y-10)*240] = 0
                ziel -=1
            if Liste[pos] == 0 and Liste[pos+10] != 255: 
                farbe = 0x0000
                pac(x,y,farbe)
                if x < 230:
                    x = x + 20
                farbe = 0xE0FF
                pac(x,y,farbe)
                pac_rechts(x,y)
                LCD.show()
                farbe = 0xE0FF
                pac(x,y,farbe)
                LCD.show()
            else:
                farbe = 0xE0FF
                pac(x,y,farbe)
                pac_rechts(x,y)
                LCD.show()
                farbe = 0xE0FF
                pac(x,y,farbe)
                LCD.show()
        if ziel < 3 and start == 1:
            LCD.fill_rect(40,30,160,80,LCD.white)
            LCD.rect(45,35,150,70,LCD.black)
            LCD.text("Ziel erreicht.",60,45,LCD.blue)
            LCD.text("mit "+str(punkte)+" Punkten",60,65,LCD.blue)
            LCD.text("10 s Pause",60,85,LCD.blue)
            LCD.show()
            ziel = 0
            step = 130
            time.sleep(10)
            start = 0
        if ((leben == 0 or restzeit == 0 )and start == 1):
            LCD.fill_rect(40,30,160,80,LCD.white)
            LCD.rect(45,35,150,70,LCD.black)
            LCD.text("Spiel zu Ende",60,45,LCD.red)
            if leben == 0 :
                LCD.text("keine Leben mehr",60,65,LCD.blue)
            if restzeit == 0 :
                LCD.text("Zeit vorbei",60,65,LCD.blue)
            LCD.text("10 s Pause",60,85,LCD.red)
            LCD.show()
            ziel = 0
            step = 130
            time.sleep(10)
            start = 0